@@ -80,6 +80,7 @@ def _filter_list(data, name_or_id, filters):
         OR
 
         A string containing a jmespath expression for further filtering.
+        Invalid filters will be ignored.
     """
     # The logger is openstack.cloud.fmmatch to allow a user/operator to
     # configure logging not to communicate about fnmatch misses
@@ -131,6 +132,17 @@ def _filter_list(data, name_or_id, filters):
         if not d:
             return False
         for key in f.keys():
+            if key not in d:
+                log.warning(
+                    "Invalid filter: %s is not an attribute of %s.%s",
+                    key,
+                    e.__class__.__module__,
+                    e.__class__.__qualname__,
+                )
+                # we intentionally skip this since the user was trying to
+                # filter on _something_, but we don't know what that
+                # _something_ was
+                raise AttributeError(key)
             if isinstance(f[key], dict):
                 if not _dict_filter(f[key], d.get(key, None)):
                     return False
@@ -140,15 +152,8 @@ def _filter_list(data, name_or_id, filters):
 
     filtered = []
     for e in data:
-        filtered.append(e)
-        for key in filters.keys():
-            if isinstance(filters[key], dict):
-                if not _dict_filter(filters[key], e.get(key, None)):
-                    filtered.pop()
-                    break
-            elif e.get(key, None) != filters[key]:
-                filtered.pop()
-                break
+        if _dict_filter(filters, e):
+            filtered.append(e)
     return filtered
 
 
@@ -168,7 +173,7 @@ def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
         Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
     """
 
-    # Sometimes in the control flow of shade, we already have an object
+    # Sometimes in the control flow of openstacksdk, we already have an object
     # fetched. Rather than then needing to pull the name or id out of that
     # object, pass it in here and rely on caching to prevent us from making
     # an additional call, it's simple enough to test to see if we got an
@@ -284,13 +289,13 @@ def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
 
 
 @contextlib.contextmanager
-def shade_exceptions(error_message=None):
-    """Context manager for dealing with shade exceptions.
+def openstacksdk_exceptions(error_message=None):
+    """Context manager for dealing with openstack exceptions.
 
     :param string error_message: String to use for the exception message
         content on non-OpenStackCloudExceptions.
 
-        Useful for avoiding wrapping shade OpenStackCloudException exceptions
+        Useful for avoiding wrapping OpenStackCloudException exceptions
         within themselves. Code called from within the context may throw such
         exceptions without having to catch and reraise them.
 
