@@ -9,15 +9,26 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
-from openstack.compute.v2 import metadata
-from openstack.image.v2 import image
+from openstack.common import metadata
+from openstack.common import tag
+from openstack.compute.v2 import flavor
+from openstack.compute.v2 import volume_attachment
 from openstack import exceptions
+from openstack.image.v2 import image
 from openstack import resource
 from openstack import utils
 
 
-class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
+CONSOLE_TYPE_ACTION_MAPPING = {
+    'novnc': 'os-getVNCConsole',
+    'xvpvnc': 'os-getVNCConsole',
+    'spice-html5': 'os-getSPICEConsole',
+    'rdp-html5': 'os-getRDPConsole',
+    'serial': 'os-getSerialConsole',
+}
+
+
+class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     resource_key = 'server'
     resources_key = 'servers'
     base_path = '/servers'
@@ -29,16 +40,39 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     allow_delete = True
     allow_list = True
 
+    # Sentinel used to differentiate API called without parameter or None
+    # Ex unshelve API can be called without an availability_zone or with
+    # availability_zone = None to unpin the az.
+    _sentinel = object()
+
     _query_mapping = resource.QueryParameters(
-        "auto_disk_config", "availability_zone",
-        "created_at", "description", "flavor",
-        "hostname", "image", "kernel_id", "key_name",
-        "launch_index", "launched_at", "locked_by", "name",
-        "node", "power_state", "progress", "project_id", "ramdisk_id",
-        "reservation_id", "root_device_name",
-        "status", "task_state", "terminated_at", "user_id",
+        "auto_disk_config",
+        "availability_zone",
+        "created_at",
+        "description",
+        "flavor",
+        "hostname",
+        "image",
+        "kernel_id",
+        "key_name",
+        "launch_index",
+        "launched_at",
+        "locked_by",
+        "name",
+        "node",
+        "power_state",
+        "progress",
+        "project_id",
+        "ramdisk_id",
+        "reservation_id",
+        "root_device_name",
+        "status",
+        "task_state",
+        "terminated_at",
+        "user_id",
         "vm_state",
-        "sort_key", "sort_dir",
+        "sort_key",
+        "sort_dir",
         access_ipv4="access_ip_v4",
         access_ipv6="access_ip_v6",
         has_config_drive="config_drive",
@@ -51,10 +85,10 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         changes_before="changes-before",
         id="uuid",
         all_projects="all_tenants",
-        **resource.TagMixin._tag_query_parameters
+        **tag.TagMixin._tag_query_parameters,
     )
 
-    _max_microversion = '2.72'
+    _max_microversion = '2.91'
 
     #: A list of dictionaries holding links relevant to this server.
     links = resource.Body('links')
@@ -73,7 +107,12 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     #: A list of an attached volumes. Each item in the list contains at least
     #: an "id" key to identify the specific volumes.
     attached_volumes = resource.Body(
-        'os-extended-volumes:volumes_attached')
+        'os-extended-volumes:volumes_attached',
+        aka='volumes',
+        type=list,
+        list_type=volume_attachment.VolumeAttachment,
+        default=[],
+    )
     #: The name of the availability zone this server is a part of.
     availability_zone = resource.Body('OS-EXT-AZ:availability_zone')
     #: Enables fine grained control of the block device mapping for an
@@ -95,13 +134,15 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     #: this server.
     flavor_id = resource.Body('flavorRef')
     #: The flavor property as returned from server.
-    # TODO(gtema): replace with flavor.Flavor addressing flavor.original_name
-    flavor = resource.Body('flavor', type=dict)
+    flavor = resource.Body('flavor', type=flavor.Flavor)
     #: Indicates whether a configuration drive enables metadata injection.
     #: Not all cloud providers enable this feature.
     has_config_drive = resource.Body('config_drive')
     #: An ID representing the host of this server.
     host_id = resource.Body('hostId')
+    #: A fault object. Only available when the server status
+    #: is ERROR or DELETED and a fault occurred.
+    fault = resource.Body('fault')
     #: The host status.
     host_status = resource.Body('host_status')
     #: The hostname set on the instance when it is booted.
@@ -119,6 +160,12 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     #: instance name template. Appears in the response for administrative users
     #: only.
     instance_name = resource.Body('OS-EXT-SRV-ATTR:instance_name')
+    #: The address to use to connect to this server from the current calling
+    #: context. This will be set to public_ipv6 if the calling host has
+    #: routable ipv6 addresses, and to private_ipv4 if the Connection was
+    #: created with private=True. Otherwise it will be set to public_ipv4.
+    interface_ip = resource.Computed('interface_ip', default='')
+
     # The locked status of the server
     is_locked = resource.Body('locked', type=bool)
     #: The UUID of the kernel image when using an AMI. Will be null if not.
@@ -132,18 +179,15 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     launch_index = resource.Body('OS-EXT-SRV-ATTR:launch_index', type=int)
     #: The timestamp when the server was launched.
     launched_at = resource.Body('OS-SRV-USG:launched_at')
-    #: Metadata stored for this server. *Type: dict*
-    metadata = resource.Body('metadata', type=dict)
+    #: The maximum number of servers to create.
+    max_count = resource.Body('max_count')
+    #: The minimum number of servers to create.
+    min_count = resource.Body('min_count')
     #: A networks object. Required parameter when there are multiple
     #: networks defined for the tenant. When you do not specify the
     #: networks parameter, the server attaches to the only network
     #: created for the current tenant.
     networks = resource.Body('networks')
-    #: The file path and contents, text only, to inject into the server at
-    #: launch. The maximum size of the file path data is 255 bytes.
-    #: The maximum limit is The number of allowed bytes in the decoded,
-    #: rather than encoded, data.
-    personality = resource.Body('personality')
     #: The power state of this server.
     power_state = resource.Body('OS-EXT-STS:power_state')
     #: While the server is building, this value represents the percentage
@@ -151,6 +195,17 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     progress = resource.Body('progress', type=int)
     #: The ID of the project this server is associated with.
     project_id = resource.Body('tenant_id')
+
+    #: The private IPv4 address of this server
+    private_v4 = resource.Computed('private_v4', default='')
+    #: The private IPv6 address of this server
+    private_v6 = resource.Computed('private_v6', default='')
+
+    #: The public IPv4 address of this server
+    public_v4 = resource.Computed('public_v4', default='')
+    #: The public IPv6 address of this server
+    public_v6 = resource.Computed('public_v6', default='')
+
     #: The UUID of the ramdisk image when using an AMI. Will be null if not.
     #: By default, it appears in the response for administrative users only.
     ramdisk_id = resource.Body('OS-EXT-SRV-ATTR:ramdisk_id')
@@ -166,8 +221,9 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     scheduler_hints = resource.Body('OS-SCH-HNT:scheduler_hints', type=dict)
     #: A list of applicable security groups. Each group contains keys for
     #: description, name, id, and rules.
-    security_groups = resource.Body('security_groups',
-                                    type=list, list_type=dict)
+    security_groups = resource.Body(
+        'security_groups', type=list, list_type=dict
+    )
     #: The UUIDs of the server groups to which the server belongs.
     #: Currently this can contain at most one entry.
     server_groups = resource.Body('server_groups', type=list)
@@ -184,7 +240,8 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     #: A list of trusted certificate IDs, that were used during image
     #: signature verification to verify the signing certificate.
     trusted_image_certificates = resource.Body(
-        'trusted_image_certificates', type=list)
+        'trusted_image_certificates', type=list
+    )
     #: Timestamp of when this server was last updated.
     updated_at = resource.Body('updated')
     #: Configuration information or scripts to use upon launch.
@@ -195,11 +252,18 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     #: The VM state of this server.
     vm_state = resource.Body('OS-EXT-STS:vm_state')
 
-    def _prepare_request(self, requires_id=True, prepend_key=True,
-                         base_path=None):
-        request = super(Server, self)._prepare_request(requires_id=requires_id,
-                                                       prepend_key=prepend_key,
-                                                       base_path=base_path)
+    def _prepare_request(
+        self,
+        requires_id=True,
+        prepend_key=True,
+        base_path=None,
+        **kwargs,
+    ):
+        request = super(Server, self)._prepare_request(
+            requires_id=requires_id,
+            prepend_key=prepend_key,
+            base_path=base_path,
+        )
 
         server_body = request.body[self.resource_key]
 
@@ -235,8 +299,18 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         # the URL used is sans any additional /detail/ part.
         url = utils.urljoin(Server.base_path, self.id, 'action')
         headers = {'Accept': ''}
+
+        # these aren't all necessary "commit" actions (i.e. updates) but it's
+        # good enough...
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
+
         response = session.post(
-            url, json=body, headers=headers, microversion=microversion)
+            url,
+            json=body,
+            headers=headers,
+            microversion=microversion,
+        )
         exceptions.raise_from_response(response)
         return response
 
@@ -248,7 +322,12 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     def get_password(self, session):
         """Get the encrypted administrator password."""
         url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
-        return session.get(url)
+
+        response = session.get(url)
+        exceptions.raise_from_response(response)
+
+        data = response.json()
+        return data.get('password')
 
     def reboot(self, session, reboot_type):
         """Reboot server where reboot_type might be 'SOFT' or 'HARD'."""
@@ -260,26 +339,37 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         body = {'forceDelete': None}
         self._action(session, body)
 
-    def rebuild(self, session, name, admin_password,
-                preserve_ephemeral=False, image=None,
-                access_ipv4=None, access_ipv6=None,
-                metadata=None, personality=None):
+    def rebuild(
+        self,
+        session,
+        image,
+        name=None,
+        admin_password=None,
+        preserve_ephemeral=None,
+        access_ipv4=None,
+        access_ipv6=None,
+        metadata=None,
+        user_data=None,
+        key_name=None,
+    ):
         """Rebuild the server with the given arguments."""
-        action = {
-            'name': name,
-            'adminPass': admin_password,
-            'preserve_ephemeral': preserve_ephemeral
-        }
-        if image is not None:
-            action['imageRef'] = resource.Resource._get_id(image)
+        action = {'imageRef': resource.Resource._get_id(image)}
+        if preserve_ephemeral is not None:
+            action['preserve_ephemeral'] = preserve_ephemeral
+        if name is not None:
+            action['name'] = name
+        if admin_password is not None:
+            action['adminPass'] = admin_password
         if access_ipv4 is not None:
             action['accessIPv4'] = access_ipv4
         if access_ipv6 is not None:
             action['accessIPv6'] = access_ipv6
         if metadata is not None:
             action['metadata'] = metadata
-        if personality is not None:
-            action['personality'] = personality
+        if user_data is not None:
+            action['user_data'] = user_data
+        if key_name is not None:
+            action['key_name'] = key_name
 
         body = {'rebuild': action}
         response = self._action(session, body)
@@ -376,7 +466,7 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
             "createBackup": {
                 "name": name,
                 "backup_type": backup_type,
-                "rotation": rotation
+                "rotation": rotation,
             }
         }
         self._action(session, body)
@@ -397,8 +487,12 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         body = {"resume": None}
         self._action(session, body)
 
-    def lock(self, session):
+    def lock(self, session, locked_reason=None):
         body = {"lock": None}
+        if locked_reason is not None:
+            body["lock"] = {
+                "locked_reason": locked_reason,
+            }
         self._action(session, body)
 
     def unlock(self, session):
@@ -435,18 +529,48 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         body = {"os-stop": None}
         self._action(session, body)
 
+    def restore(self, session):
+        body = {"restore": None}
+        self._action(session, body)
+
     def shelve(self, session):
         body = {"shelve": None}
         self._action(session, body)
 
-    def unshelve(self, session):
-        body = {"unshelve": None}
+    def shelve_offload(self, session):
+        body = {"shelveOffload": None}
+        self._action(session, body)
+
+    def unshelve(self, session, availability_zone=_sentinel, host=None):
+        """
+        Unshelve -- Unshelve the server.
+
+        :param availability_zone: If specified the instance will be unshelved
+            to the availability_zone.
+            If None is passed the instance defined availability_zone is unpin
+            and the instance will be scheduled to any availability_zone (free
+            scheduling).
+            If not specified the instance will be unshelved to either its
+            defined availability_zone or any availability_zone
+            (free scheduling).
+        :param host: If specified the host to unshelve the instance.
+        """
+        data = {}
+        if host:
+            data["host"] = host
+        if availability_zone is None or isinstance(availability_zone, str):
+            data["availability_zone"] = availability_zone
+        body = {'unshelve': data or None}
         self._action(session, body)
 
     def migrate(self, session):
         body = {"migrate": None}
         self._action(session, body)
 
+    def trigger_crash_dump(self, session):
+        body = {"trigger_crash_dump": None}
+        self._action(session, body)
+
     def get_console_output(self, session, length=None):
         body = {"os-getConsoleOutput": {}}
         if length is not None:
@@ -454,24 +578,44 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         resp = self._action(session, body)
         return resp.json()
 
-    def live_migrate(self, session, host, force, block_migration,
-                     disk_over_commit=False):
+    def get_console_url(self, session, console_type):
+        action = CONSOLE_TYPE_ACTION_MAPPING.get(console_type)
+        if not action:
+            raise ValueError("Unsupported console type %s" % console_type)
+        body = {action: {'type': console_type}}
+        resp = self._action(session, body)
+        return resp.json().get('console')
+
+    def live_migrate(
+        self,
+        session,
+        host,
+        force,
+        block_migration,
+        disk_over_commit=False,
+    ):
         if utils.supports_microversion(session, '2.30'):
             return self._live_migrate_30(
-                session, host,
-                force=force,
-                block_migration=block_migration)
-        elif utils.supports_microversion(session, '2.25'):
-            return self._live_migrate_25(
-                session, host,
-                force=force,
-                block_migration=block_migration)
-        else:
-            return self._live_migrate(
-                session, host,
+                session,
+                host,
                 force=force,
                 block_migration=block_migration,
-                disk_over_commit=disk_over_commit)
+            )
+        elif utils.supports_microversion(session, '2.25'):
+            return self._live_migrate_25(
+                session,
+                host,
+                force=force,
+                block_migration=block_migration,
+            )
+        else:
+            return self._live_migrate(
+                session,
+                host,
+                force=force,
+                block_migration=block_migration,
+                disk_over_commit=disk_over_commit,
+            )
 
     def _live_migrate_30(self, session, host, force, block_migration):
         microversion = '2.30'
@@ -484,7 +628,10 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
             if force:
                 body['force'] = force
         self._action(
-            session, {'os-migrateLive': body}, microversion=microversion)
+            session,
+            {'os-migrateLive': body},
+            microversion=microversion,
+        )
 
     def _live_migrate_25(self, session, host, force, block_migration):
         microversion = '2.25'
@@ -501,12 +648,22 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
                     " possible to disable. It is recommended to not use 'host'"
                     " at all on this cloud as it is inherently unsafe, but if"
                     " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks.")
+                    " is clear you understand the risks."
+                )
         self._action(
-            session, {'os-migrateLive': body}, microversion=microversion)
+            session,
+            {'os-migrateLive': body},
+            microversion=microversion,
+        )
 
-    def _live_migrate(self, session, host, force, block_migration,
-                      disk_over_commit):
+    def _live_migrate(
+        self,
+        session,
+        host,
+        force,
+        block_migration,
+        disk_over_commit,
+    ):
         microversion = None
         body = {
             'host': None,
@@ -514,7 +671,8 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         if block_migration == 'auto':
             raise ValueError(
                 "Live migration on this cloud does not support 'auto' as"
-                " a parameter to block_migration, but only True and False.")
+                " a parameter to block_migration, but only True and False."
+            )
         body['block_migration'] = block_migration or False
         body['disk_over_commit'] = disk_over_commit or False
         if host:
@@ -526,9 +684,27 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
                     " possible to disable. It is recommended to not use 'host'"
                     " at all on this cloud as it is inherently unsafe, but if"
                     " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks.")
+                    " is clear you understand the risks."
+                )
         self._action(
-            session, {'os-migrateLive': body}, microversion=microversion)
+            session,
+            {'os-migrateLive': body},
+            microversion=microversion,
+        )
+
+    def fetch_topology(self, session):
+        utils.require_microversion(session, 2.78)
+
+        url = utils.urljoin(Server.base_path, self.id, 'topology')
+
+        response = session.get(url)
+
+        exceptions.raise_from_response(response)
+
+        try:
+            return response.json()
+        except ValueError:
+            pass
 
     def fetch_security_groups(self, session):
         """Fetch security groups of a server.
