@@ -27,6 +27,7 @@ from openstack.image.v2 import task as _task
 from openstack import proxy
 from openstack import resource
 from openstack import utils
+from openstack import warnings as os_warnings
 
 # Rackspace returns this for intermittent import errors
 _IMAGE_ERROR_396 = "Image cannot be imported. Error code: '396'"
@@ -54,6 +55,8 @@ class Proxy(proxy.Proxy):
         "image": _image.Image,
         "image_member": _member.Member,
         "metadef_namespace": _metadef_namespace.MetadefNamespace,
+        "metadef_resource_type": _metadef_resource_type.MetadefResourceType,
+        "metadef_resource_type_association": _metadef_resource_type.MetadefResourceTypeAssociation,  # noqa
         "schema": _schema.Schema,
         "info_import": _si.Import,
         "info_store": _si.Store,
@@ -500,7 +503,10 @@ class Proxy(proxy.Proxy):
         :returns: The results of image creation
         :rtype: :class:`~openstack.image.v2.image.Image`
         """
-        warnings.warn("upload_image is deprecated. Use create_image instead.")
+        warnings.warn(
+            "upload_image is deprecated. Use create_image instead.",
+            os_warnings.OpenStackDeprecationWarning,
+        )
         # container_format and disk_format are required to be set
         # on the image by the time upload_image is called, but they're not
         # required by the _create call. Enforce them here so that we don't
@@ -778,7 +784,7 @@ class Proxy(proxy.Proxy):
         *,
         stream=False,
         output=None,
-        chunk_size=1024,
+        chunk_size=1024 * 1024,
     ):
         """Download an image
 
@@ -802,7 +808,7 @@ class Proxy(proxy.Proxy):
             When ``False``, return the entire contents of the response.
         :param output: Either a file object or a path to store data into.
         :param int chunk_size: size in bytes to read from the wire and buffer
-            at one time. Defaults to 1024
+            at one time. Defaults to 1024 * 1024 = 1 MiB
 
         :returns: When output is not given - the bytes comprising the given
             Image when stream is False, otherwise a :class:`requests.Response`
