@@ -1030,7 +1030,7 @@ class Proxy(proxy.Proxy):
                 raise ValueError('path must at least contain /v1/a/c/')
             else:
                 raise ValueError(
-                    'path must be full path to an object' ' e.g. /v1/a/c/o'
+                    'path must be full path to an object e.g. /v1/a/c/o'
                 )
 
         standard_methods = ['GET', 'PUT', 'HEAD', 'POST', 'DELETE']
@@ -1130,7 +1130,13 @@ class Proxy(proxy.Proxy):
         identified_resources=None,
         filters=None,
         resource_evaluation_fn=None,
+        skip_resources=None,
     ):
+        if self.should_skip_resource_cleanup(
+            "container", skip_resources
+        ) or self.should_skip_resource_cleanup("object", skip_resources):
+            return
+
         is_bulk_delete_supported = False
         bulk_delete_max_per_request = None
         try:
