@@ -100,14 +100,14 @@ class Proxy(adapter.Adapter, Generic[T]):
         influxdb_config=None,
         influxdb_client=None,
         *args,
-        **kwargs
+        **kwargs,
     ):
         # NOTE(dtantsur): keystoneauth defaults retriable_status_codes to None,
         # override it with a class-level value.
         kwargs.setdefault(
             'retriable_status_codes', self.retriable_status_codes
         )
-        super(Proxy, self).__init__(session=session, *args, **kwargs)
+        super().__init__(session=session, *args, **kwargs)
         self._statsd_client = statsd_client
         self._statsd_prefix = statsd_prefix
         self._prometheus_counter = prometheus_counter
@@ -144,7 +144,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         connect_retries=1,
         global_request_id=None,
         *args,
-        **kwargs
+        **kwargs,
     ):
         conn = self._get_connection()
         if not global_request_id:
@@ -173,14 +173,14 @@ class Proxy(adapter.Adapter, Generic[T]):
                 # Get from cache or execute and cache
                 response = conn._cache.get_or_create(
                     key=key,
-                    creator=super(Proxy, self).request,
+                    creator=super().request,
                     creator_args=(
                         [url, method],
                         dict(
                             connect_retries=connect_retries,
                             raise_exc=raise_exc,
                             global_request_id=global_request_id,
-                            **kwargs
+                            **kwargs,
                         ),
                     ),
                     expiration_time=expiration_time,
@@ -190,13 +190,13 @@ class Proxy(adapter.Adapter, Generic[T]):
                 # asked for cache bypass
                 self._invalidate_cache(conn, key_prefix)
                 # Pass through the API request bypassing cache
-                response = super(Proxy, self).request(
+                response = super().request(
                     url,
                     method,
                     connect_retries=connect_retries,
                     raise_exc=raise_exc,
                     global_request_id=global_request_id,
-                    **kwargs
+                    **kwargs,
                 )
 
             for h in response.history:
@@ -623,7 +623,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         requires_id=True,
         base_path=None,
         skip_cache=False,
-        **attrs
+        **attrs,
     ):
         """Fetch a resource
 
@@ -665,7 +665,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         paginated=True,
         base_path=None,
         jmespath_filters=None,
-        **attrs
+        **attrs,
     ) -> Generator[T, None, None]:
         """List a resource
 
@@ -736,6 +736,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         identified_resources=None,
         filters=None,
         resource_evaluation_fn=None,
+        skip_resources=None,
     ):
         return None
 
@@ -814,7 +815,23 @@ class Proxy(adapter.Adapter, Generic[T]):
         else:
             return False
 
+    def should_skip_resource_cleanup(self, resource=None, skip_resources=None):
+        if resource is None or skip_resources is None:
+            return False
 
+        resource_name = f"{self.service_type.replace('-', '_')}.{resource}"
+
+        if resource_name in skip_resources:
+            self.log.debug(
+                f"Skipping resource {resource_name} " "in project cleanup"
+            )
+            return True
+
+        return False
+
+
+# TODO(stephenfin): Remove this and all users. Use of this generally indicates
+# a missing Resource type.
 def _json_response(response, result_key=None, error_message=None):
     """Temporary method to use to bridge from ShadeAdapter to SDK calls."""
     exceptions.raise_from_response(response, error_message=error_message)
@@ -832,11 +849,3 @@ def _json_response(response, result_key=None, error_message=None):
     except JSONDecodeError:
         return response
     return result_json
-
-
-class _ShadeAdapter(Proxy):
-    """Wrapper for shade methods that expect json unpacking."""
-
-    def request(self, url, method, error_message=None, **kwargs):
-        response = super(_ShadeAdapter, self).request(url, method, **kwargs)
-        return _json_response(response, error_message=error_message)
