@@ -17,6 +17,10 @@ from openstack.shared_file_system.v2 import (
 )
 from openstack.shared_file_system.v2 import limit as _limit
 from openstack.shared_file_system.v2 import share as _share
+from openstack.shared_file_system.v2 import share_group as _share_group
+from openstack.shared_file_system.v2 import (
+    share_group_snapshot as _share_group_snapshot,
+)
 from openstack.shared_file_system.v2 import (
     share_access_rule as _share_access_rule,
 )
@@ -50,6 +54,8 @@ class Proxy(proxy.Proxy):
         "share_instance": _share_instance.ShareInstance,
         "share_export_locations": _share_export_locations.ShareExportLocation,
         "share_access_rule": _share_access_rule.ShareAccessRule,
+        "share_group": _share_group.ShareGroup,
+        "share_group_snapshot": _share_group_snapshot.ShareGroupSnapshot,
     }
 
     def availability_zones(self):
@@ -211,6 +217,106 @@ class Proxy(proxy.Proxy):
         elif new_size < res.size and no_shrink is not True:
             res.shrink_share(self, new_size)
 
+    def share_groups(self, **query):
+        """Lists all share groups.
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the share groups being returned.  Available parameters include:
+
+            * status: Filters by a share group status.
+            * name: The user defined name of the resource to filter resources
+                by.
+            * description: The user defined description text that can be used
+                to filter resources.
+            * project_id: The project ID of the user or service.
+            * share_server_id: The UUID of the share server.
+            * snapshot_id: The UUID of the share’s base snapshot to filter
+                the request based on.
+            * host: The host name for the back end.
+            * share_network_id: The UUID of the share network to filter
+                resources by.
+            * share_group_type_id: The share group type ID to filter
+                share groups.
+            * share_group_snapshot_id: The source share group snapshot ID to
+                list the share group.
+            * share_types: A list of one or more share type IDs. Allows
+                filtering share groups.
+            * limit: The maximum number of share groups members to return.
+            * offset: The offset to define start point of share or share
+                group listing.
+            * sort_key: The key to sort a list of shares.
+            * sort_dir: The direction to sort a list of shares
+            * name~: The name pattern that can be used to filter shares,
+                share snapshots, share networks or share groups.
+            * description~: The description pattern that can be used to
+                filter shares, share snapshots, share networks or share groups.
+
+        :returns: A generator of manila share group resources
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_group.ShareGroup`
+        """
+        return self._list(_share_group.ShareGroup, **query)
+
+    def get_share_group(self, share_group_id):
+        """Lists details for a share group.
+
+        :param share: The ID of the share group to get
+        :returns: Details of the identified share group
+        :rtype: :class:`~openstack.shared_file_system.v2.
+                                    share_group.ShareGroup`
+        """
+        return self._get(_share_group.ShareGroup, share_group_id)
+
+    def find_share_group(self, name_or_id, ignore_missing=True):
+        """Finds a single share group
+
+        :param name_or_id: The name or ID of a share group.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, None will be returned when
+            attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.shared_file_system.v2.
+                                        share_group.ShareGroup`
+                                        or None
+        """
+        return self._find(
+            _share_group.ShareGroup, name_or_id, ignore_missing=ignore_missing
+        )
+
+    def create_share_group(self, **attrs):
+        """Creates a share group from attributes
+
+        :returns: Details of the new share group
+        :rtype: :class:`~openstack.shared_file_system.v2.
+                                    share_group.ShareGroup`
+        """
+        return self._create(_share_group.ShareGroup, **attrs)
+
+    def update_share_group(self, share_group_id, **kwargs):
+        """Updates details of a single share group
+
+        :param share: The ID of the share group
+        :returns: Updated details of the identified share group
+        :rtype: :class:`~openstack.shared_file_system.v2.
+                                    share_group.ShareGroup`
+        """
+        return self._update(_share_group.ShareGroup, share_group_id, **kwargs)
+
+    def delete_share_group(self, share_group_id, ignore_missing=True):
+        """Deletes a single share group
+
+        :param share: The ID of the share group
+        :returns: Result of the "delete" on share group
+        :rtype: :class:`~openstack.shared_file_system.v2.
+                                    share_group.ShareGroup`
+        """
+        return self._delete(
+            _share_group.ShareGroup,
+            share_group_id,
+            ignore_missing=ignore_missing,
+        )
+
     def wait_for_status(
         self, res, status='active', failures=None, interval=2, wait=120
     ):
@@ -710,7 +816,128 @@ class Proxy(proxy.Proxy):
         :param access_id: The id of the access rule to get
         :param share_id: The ID of the share
 
-        :rtype: ``None``
+        :rtype: ``requests.models.Response`` HTTP response from internal
+            requests client
         """
         res = self._get_resource(_share_access_rule.ShareAccessRule, access_id)
-        res.delete(self, share_id, ignore_missing=ignore_missing)
+        return res.delete(self, share_id, ignore_missing=ignore_missing)
+
+    def share_group_snapshots(self, details=True, **query):
+        """Lists all share group snapshots.
+
+        :param kwargs query: Optional query parameters to be sent
+            to limit the share group snapshots being returned.
+            Available parameters include:
+
+            * project_id: The ID of the project that owns the resource.
+            * name: The user defined name of the resource to filter resources.
+            * description: The user defined description text that can be used
+              to filter resources.
+            * status: Filters by a share status
+            * share_group_id: The UUID of a share group to filter resource.
+            * limit: The maximum number of share group snapshot members
+              to return.
+            * offset: The offset to define start point of share or
+              share group listing.
+            * sort_key: The key to sort a list of shares.
+            * sort_dir: The direction to sort a list of shares. A valid
+              value is asc, or desc.
+
+        :returns: Details of share group snapshots resources
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_group_snapshot.ShareGroupSnapshot`
+        """
+        base_path = '/share-group-snapshots/detail' if details else None
+        return self._list(
+            _share_group_snapshot.ShareGroupSnapshot,
+            base_path=base_path,
+            **query
+        )
+
+    def share_group_snapshot_members(self, group_snapshot_id):
+        """Lists all share group snapshots members.
+
+        :param group_snapshot_id: The ID of the group snapshot to get
+        :returns: List of the share group snapshot members, which are
+            share snapshots.
+        :rtype: dict containing attributes of the share snapshot members.
+        """
+        res = self._get(
+            _share_group_snapshot.ShareGroupSnapshot,
+            group_snapshot_id,
+        )
+        response = res.get_members(self)
+        return response
+
+    def get_share_group_snapshot(self, group_snapshot_id):
+        """Show share group snapshot details
+
+        :param group_snapshot_id: The ID of the group snapshot to get
+        :returns: Details of the group snapshot
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_group_snapshot.ShareGroupSnapshot`
+        """
+        return self._get(
+            _share_group_snapshot.ShareGroupSnapshot, group_snapshot_id
+        )
+
+    def create_share_group_snapshot(self, share_group_id, **attrs):
+        """Creates a point-in-time snapshot copy of a share group.
+
+        :returns: Details of the new snapshot
+        :param dict attrs: Attributes which will be used to create
+            a :class:`~openstack.shared_file_system.v2.
+            share_group_snapshots.ShareGroupSnapshots`,
+        :param 'share_group_id': ID of the share group to have the snapshot
+            taken.
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_group_snapshot.ShareGroupSnapshot`
+        """
+        return self._create(
+            _share_group_snapshot.ShareGroupSnapshot,
+            share_group_id=share_group_id,
+            **attrs
+        )
+
+    def reset_share_group_snapshot_status(self, group_snapshot_id, status):
+        """Reset share group snapshot state.
+
+        :param group_snapshot_id: The ID of the share group snapshot to reset
+        :param status: The state of the share group snapshot to be set, A
+            valid value is "creating", "error", "available", "deleting",
+            "error_deleting".
+        :rtype: ``None``
+        """
+        res = self._get(
+            _share_group_snapshot.ShareGroupSnapshot, group_snapshot_id
+        )
+        res.reset_status(self, status)
+
+    def update_share_group_snapshot(self, group_snapshot_id, **attrs):
+        """Updates a share group snapshot.
+
+        :param group_snapshot_id: The ID of the share group snapshot to update
+        :param dict attrs: The attributes to update on the share group snapshot
+        :returns: the updated share group snapshot
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_group_snapshot.ShareGroupSnapshot`
+        """
+        return self._update(
+            _share_group_snapshot.ShareGroupSnapshot,
+            group_snapshot_id,
+            **attrs
+        )
+
+    def delete_share_group_snapshot(
+        self, group_snapshot_id, ignore_missing=True
+    ):
+        """Deletes a share group snapshot.
+
+        :param group_snapshot_id: The ID of the share group snapshot to delete
+        :rtype: ``None``
+        """
+        self._delete(
+            _share_group_snapshot.ShareGroupSnapshot,
+            group_snapshot_id,
+            ignore_missing=ignore_missing,
+        )
